/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const char CB_EXCEPTIONS[] =
R"__cs_cb(    public class InvalidIOException : InvalidOperationException {}
    public class InvalidIDException : InvalidOperationException {}
    public class PermissionDeniedException : InvalidOperationException {}
    public class InvalidProtocolException : InvalidOperationException {}
    public class NotConnectedSocketException : InvalidOperationException {}
    public class InvalidCallbackException : InvalidOperationException {}

)__cs_cb";

const char CB_CALLBACK_BASE[] =
R"__cs_cb(
            public abstract class CallbackBase
            {
                internal int Id;
                internal int SeqId;
                internal bool Once;
                private static int _seqNum = 0;

                public string Tag
                {
                    get
                    {
                        return Id.ToString() + "::" + SeqId.ToString();
                    }
                }

                public CallbackBase(int delegateId, bool once)
                {
                    Id = delegateId;
                    SeqId = _seqNum++;
                    Once = once;
                }

                internal virtual void OnReceivedEvent(IntPtr port) {}

                internal static void Serialize(IntPtr h, CallbackBase param)
                {
                    Interop.LibRPCPort.Parcel.WriteInt32(h, param.Id);
                    Interop.LibRPCPort.Parcel.WriteInt32(h, param.SeqId);
                    Interop.LibRPCPort.Parcel.WriteBool(h, param.Once);
                }

                internal static void Deserialize(IntPtr h, CallbackBase param)
                {
                    Interop.LibRPCPort.Parcel.ReadInt32(h, out var id);
                    param.Id = id;
                    Interop.LibRPCPort.Parcel.ReadInt32(h, out var seqId);
                    param.SeqId = seqId;
                    Interop.LibRPCPort.Parcel.ReadBool(h, out var once);
                    param.Once = once;
                }
            }

)__cs_cb";

const char CB_CALLBACK_CTOR_PROXY[] =
R"__cs_cb(                public $$(bool once = false) : base((int)DelegateId.$$, once)
                {
                }
)__cs_cb";

const char CB_CALLBACK_CTOR_STUB[] =
R"__cs_cb(                internal $$(IntPtr port, WeakReference service) : base((int)DelegateId.$$, false)
                {
                    _port = port;
                    _service = service;
                }
)__cs_cb";

const char CB_CALLBACK_STUB_MEMBERS[] =
R"__cs_cb(                private IntPtr _port;
                private WeakReference _service;
                private bool _valid = true;

)__cs_cb";

const char CB_CALLBACK_INVOKE_METHOD[] =
R"__cs_cb(
                public void Invoke($$)
                {
                    if (_port == IntPtr.Zero || _service == null)
                        throw new NotConnectedSocketException();
                    if (!_service.IsAlive)
                        throw new InvalidProtocolException();
                    if (Once && !_valid)
                        throw new InvalidCallbackException();

                    Interop.LibRPCPort.Parcel.Create(out IntPtr p);
                    Interop.LibRPCPort.Parcel.WriteInt32(p, (int)MethodId.__Callback);
                    Serialize(p, this);
$$
                    // Send
                    Interop.LibRPCPort.Parcel.Send(p, _port);
                    Interop.LibRPCPort.Parcel.Destroy(p);
                    _valid = false;
                }
)__cs_cb";

const char CB_PROPERTY[] = R"__cs_cb(        public $$ $$ { get; set; }
)__cs_cb";

const char CB_CTOR[] = R"__cs_cb(
        public $$()
        {
$$
        }
)__cs_cb";

const char CB_WRITE_BUNDLE[] =
R"__cs_cb(
                if (param.$$ != null)
                {
                    Interop.LibRPCPort.Parcel.WriteBundle(h, param.$$.SafeBundleHandle.DangerousGetHandle());
                }
                else
                {
                    Interop.LibRPCPort.Parcel.WriteBundle(h, new Bundle().SafeBundleHandle.DangerousGetHandle());
                }
)__cs_cb";
