/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const char CB_DATA_MEMBERS[] =
R"__cs_cb(            public event EventHandler Connected;
            public event EventHandler Disconnected;
            public event EventHandler Rejected;

            private IntPtr _proxy;
            private IntPtr _port;
            private bool _online = false;
            private string _appId;
            private Object _lock = new Object();
            private List<CallbackBase> _delegateList = new List<CallbackBase>();
            private Interop.LibRPCPort.Proxy.ConnectedEventCallback _connectedEventCallback;
            private Interop.LibRPCPort.Proxy.DisconnectedEventCallback _disconnectedEventCallback;
            private Interop.LibRPCPort.Proxy.RejectedEventCallback _rejectedEventCallback;
            private Interop.LibRPCPort.Proxy.ReceivedEventCallback _receivedEventCallback;
)__cs_cb";

const char CB_EVENT_METHODS[] =
R"__cs_cb(
            private void OnConnectedEvent(string endPoint, string port_name, IntPtr port, IntPtr data)
            {
                _port = port;
                _online = true;
                Connected?.Invoke(this, null);
            }

            private void OnDisconnectedEvent(string endPoint, string port_name, IntPtr data)
            {
                Disconnected?.Invoke(this, null);
            }

            private void OnRejectedEvent(string endPoint, string port_name, IntPtr data)
            {
                Rejected?.Invoke(this, null);
            }

            private void ProcessReceivedEvent(IntPtr parcel)
            {
                Interop.LibRPCPort.Parcel.ReadInt32(parcel, out int id);
                Interop.LibRPCPort.Parcel.ReadInt32(parcel, out int seqId);
                Interop.LibRPCPort.Parcel.ReadBool(parcel, out bool once);

                foreach (var i in _delegateList)
                {
                    if ((int)i.Id == id && i.SeqId == seqId)
                    {
                        i.OnReceivedEvent(parcel);
                        if (i.Once)
                            _delegateList.Remove(i);
                        break;
                    }
                }
            }

            private void OnReceivedEvent(string endPoint, string port_name, IntPtr data)
            {
                IntPtr parcel_received;
                lock (_lock)
                {
                    if (Interop.LibRPCPort.Parcel.CreateFromPort(out parcel_received, _port) != 0)
                        return;
                }

                Interop.LibRPCPort.Parcel.ReadInt32(parcel_received, out int cmd);
                if (cmd != (int)MethodId.__Callback)
                {
                    Interop.LibRPCPort.Parcel.Destroy(parcel_received);
                    throw new InvalidProtocolException();
                }

                ProcessReceivedEvent(parcel_received);
                Interop.LibRPCPort.Parcel.Destroy(parcel_received);
            }

            private void ConsumeCommand(out IntPtr parcel, IntPtr port)
            {
                do
                {
                    var ret = Interop.LibRPCPort.Parcel.CreateFromPort(out IntPtr p, port);

                    if (ret != Interop.LibRPCPort.ErrorCode.None)
                        break;
                    Interop.LibRPCPort.Parcel.ReadInt32(p, out int cmd);
                    if (cmd == (int)MethodId.__Result)
                    {
                        parcel = p;
                        return;
                    }

                    ProcessReceivedEvent(p);
                    Interop.LibRPCPort.Parcel.Destroy(p);
                    parcel = IntPtr.Zero;
                } while (true);
                parcel = IntPtr.Zero;
            }
)__cs_cb";

const char CB_CONNECT_METHOD[] =
R"__cs_cb(
            /// <summary>
            /// Connects to the service app.
            /// </summary>
            /// <privilege>http://tizen.org/privilege/appmanager.launch</privilege>
            /// <privilege>http://tizen.org/privilege/datasharing</privilege>
            /// <exception cref="InvalidIDException">
            /// Thrown when the appid to connect is invalid.
            /// </exception>
            /// <exception cref="InvalidIOException">
            /// Thrown when internal I/O error happen.
            /// </exception>
            /// <exception cref="PermissionDeniedException">
            /// Thrown when the permission is denied.
            /// </exception>
            /// <remark> If you want to use this method, you must add privileges.</remark>
            public void Connect()
            {
                if (Interop.LibRPCPort.Proxy.Create(out _proxy) != Interop.LibRPCPort.ErrorCode.None)
                    throw new InvalidIOException();
                _connectedEventCallback = new Interop.LibRPCPort.Proxy.ConnectedEventCallback(OnConnectedEvent);
                _disconnectedEventCallback = new Interop.LibRPCPort.Proxy.DisconnectedEventCallback(OnDisconnectedEvent);
                _rejectedEventCallback = new Interop.LibRPCPort.Proxy.RejectedEventCallback(OnRejectedEvent);
                _receivedEventCallback = new Interop.LibRPCPort.Proxy.ReceivedEventCallback(OnReceivedEvent);
                Interop.LibRPCPort.Proxy.AddConnectedEventCb(_proxy, _connectedEventCallback, IntPtr.Zero);
                Interop.LibRPCPort.Proxy.AddDisconnectedEventCb(_proxy, _disconnectedEventCallback, IntPtr.Zero);
                Interop.LibRPCPort.Proxy.AddRejectedEventCb(_proxy, _rejectedEventCallback, IntPtr.Zero);
                Interop.LibRPCPort.Proxy.AddReceivedEventCb(_proxy, _receivedEventCallback, IntPtr.Zero);

                var err = Interop.LibRPCPort.Proxy.Connect(_proxy, _appId, "$$");
                switch (err)
                {
                    case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                        throw new InvalidIDException();
                    case Interop.LibRPCPort.ErrorCode.PermissionDenied:
                        throw new PermissionDeniedException();
                    case Interop.LibRPCPort.ErrorCode.IoError:
                        throw new InvalidIOException();
                }
            }

            /// <summary>
            /// Disposes delegate objects in this interface
            /// </summary>
            /// <param name="tag">The tag string from delegate object</param>
            void DisposeCallback(string tag)
            {
                foreach (var i in _delegateList)
                {
                    if (i.Tag.Equals(tag))
                    {
                        _delegateList.Remove(i);
                        return;
                    }
                }
            }
)__cs_cb";

const char CB_DISPOSABLE[] =
R"__cs_cb(
            #region IDisposable Support
            private bool disposedValue = false;

            protected virtual void Dispose(bool disposing)
            {
                if (!disposedValue)
                {
                    if (disposing)
                    {
                    }

                    if (_proxy != IntPtr.Zero)
                        Interop.LibRPCPort.Proxy.Destroy(_proxy);

                    disposedValue = true;
                }
            }

            ~$$() {
                Dispose(false);
            }

            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }
            #endregion
)__cs_cb";

const char CB_INVOCATION_PRE[] =
R"__cs_cb(                if (!_online)
                    throw new NotConnectedSocketException();

                Interop.LibRPCPort.Parcel.Create(out IntPtr p);
)__cs_cb";

const char CB_INVOCATION_MID[] =
R"__cs_cb(                    // Send
                    Interop.LibRPCPort.Parcel.Send(p, _port);
)__cs_cb";
