/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_STUB_GEN_H_
#define IDLC_CS_GEN_STUB_GEN_H_

#include <memory>
#include <string>

#include "idlc/cs_gen/cs_gen_base.h"

namespace tidl {

class CsStubGen : public CsGeneratorBase {
 public:
  explicit CsStubGen(std::shared_ptr<Document> doc);
  virtual ~CsStubGen() = default;

  void OnInitGen(std::ofstream& stream) override;
  void OnFiniGen(std::ofstream& stream) override;

 private:
  void GenNamespace(std::ofstream& stream);
  void GenInterop(std::ofstream& stream);
  void GenInterfaces(std::ofstream& stream);
  void GenInterface(std::ofstream& stream, const Interface& iface);
  void GenServiceBase(std::ofstream& stream, const Interface& iface);
  void GenReceivedEvent(std::ofstream& stream, const Interface& iface);
  void GenConnectedEvent(std::ofstream& stream);
  void GenDisconnectedEvent(std::ofstream& stream);
  void GenCtor(std::ofstream& stream, const Interface& iface);
  void GenCommonMethods(std::ofstream& stream);
  void GenDeclarations(std::ofstream& stream, const Declarations& decls);
  void GenInvocation(std::ofstream& stream, const Declaration& decl);
  void GenDisposable(std::ofstream& stream, const Interface& iface);
};

}  // namespace tidl

#endif  // IDLC_CS_GEN_STUB_GEN_H_
