/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const char CB_DATA_MEMBERS[] =
R"__cs_cb(            private IntPtr _stub;
            private List<ServiceBase> _services = new List<ServiceBase>();
            private Type _serviceType;
            private Interop.LibRPCPort.Stub.ConnectedEventCallback _connectedEventCallback;
            private Interop.LibRPCPort.Stub.DisconnectedEventCallback _disconnectedEventCallback;
            private Interop.LibRPCPort.Stub.ReceivedEventCallback _receivedEventCallback;
)__cs_cb";

const char CB_SERVICE_BASE_FRONT[] =
R"__cs_cb(
            public abstract class ServiceBase
            {
                /// <summary>
                /// The client app ID
                /// </summary>
                public string Sender
                {
                    get; internal set;
                }

                /// <summary>
                /// The client instance ID
                /// </summary>
                public string Instance
                {
                    get; internal set;
                }

                protected ServiceBase()
                {
                }

                /// <summary>
                /// This method will be called when the client is connected
                /// </summary>
                public abstract void OnCreate();

                /// <summary>
                /// This method will be called when the client is disconnected
                /// </summary>
                public abstract void OnTerminate();
)__cs_cb";

const char CB_ON_RECEIVED_EVENT_FRONT[] =
R"__cs_cb(
            private int OnReceivedEvent(string sender, string instance, IntPtr port, IntPtr data)
            {
                var ret = Interop.LibRPCPort.Parcel.CreateFromPort(out IntPtr p, port);

                if (ret != Interop.LibRPCPort.ErrorCode.None)
                    return -1;

                Interop.LibRPCPort.Parcel.Create(out IntPtr result);
                Interop.LibRPCPort.Parcel.ReadInt32(p, out int cmd);
                ServiceBase b = null;

                foreach (var i in  _services)
                {
                    if (i.Instance.Equals(instance))
                    {
                        b = i;
                        break;
                    }
                }

                if (b == null)
                    return -1;

                switch ((MethodId)cmd)
                {
)__cs_cb";

const char CB_ON_RECEIVED_EVENT_BACK[] =
R"__cs_cb(
                    default:
                        return -1;
                }

                Interop.LibRPCPort.Parcel.Destroy(p);
                Interop.LibRPCPort.Parcel.Destroy(result);

                return 0;
            }
)__cs_cb";

const char CB_ON_CONNECTED_EVENT[] =
R"__cs_cb(
            private void OnConnectedEvent(string sender, string instance, IntPtr data)
            {
                ServiceBase s = Activator.CreateInstance(_serviceType) as ServiceBase;
                s.Sender = sender;
                s.Instance = instance;
                s.OnCreate();
                _services.Add(s);
            }
)__cs_cb";

const char CB_ON_DISCONNECTED_EVENT[] =
  R"__cs_cb(
            private void OnDisconnectedEvent(string sender, string instance, IntPtr data)
            {
                foreach (var i in _services)
                {
                    if (i.Instance.Equals(instance))
                    {
                        i.OnTerminate();
                        _services.Remove(i);
                        break;
                    }
                }
            }
)__cs_cb";

const char CB_CTOR_FRONT[] =
R"__cs_cb(
            public $$()
            {
                if (Interop.LibRPCPort.Stub.Create(out _stub, "$$") != Interop.LibRPCPort.ErrorCode.None)
                    throw new InvalidIOException();
                _connectedEventCallback = new Interop.LibRPCPort.Stub.ConnectedEventCallback(OnConnectedEvent);
                _disconnectedEventCallback = new Interop.LibRPCPort.Stub.DisconnectedEventCallback(OnDisconnectedEvent);
                _receivedEventCallback = new Interop.LibRPCPort.Stub.ReceivedEventCallback(OnReceivedEvent);
                Interop.LibRPCPort.Stub.AddReceivedEventCb(_stub, _receivedEventCallback, IntPtr.Zero);
                Interop.LibRPCPort.Stub.AddConnectedEventCb(_stub, _connectedEventCallback, IntPtr.Zero);
                Interop.LibRPCPort.Stub.AddDisconnectedEventCb(_stub, _disconnectedEventCallback, IntPtr.Zero);
)__cs_cb";

const char CB_COMMON_METHODS[] =
R"__cs_cb(
            /// <summary>
            /// Listens to client apps
            /// </summary>
            /// <param name="serviceType">The type object for making service instances</param>
            /// <exception cref="InvalidIOException">
            /// Thrown when internal I/O error happen.
            /// </exception>
            /// <exception cref="ArgumentException">
            /// Thrown when serviceType is invalid.
            /// </exception>
            public void Listen(Type serviceType)
            {
                if (!typeof(ServiceBase).IsAssignableFrom(serviceType))
                    throw new ArgumentException("Invalid type");
                _serviceType = serviceType;
                var err = Interop.LibRPCPort.Stub.Listen(_stub);
                switch (err)
                {
                    case Interop.LibRPCPort.ErrorCode.InvalidParameter:
                    case Interop.LibRPCPort.ErrorCode.IoError:
                        throw new InvalidIOException();
                }
            }

            /// <summary>
            /// Gets service objects which are connected
            /// </summary>
            /// <returns>The enumerable service objects which are connected</returns>
            public IEnumerable<ServiceBase> GetServices()
            {
                return _services;
            }
)__cs_cb";

const char CB_DISPOSABLE[] =
R"__cs_cb(
            #region IDisposable Support
            private bool disposedValue = false;

            private void Dispose(bool disposing)
            {
                if (!disposedValue)
                {
                    if (disposing)
                    {
                    }

                    foreach (var i in _services)
                    {
                        i.OnTerminate();
                    }

                    _services = null;
                    if (_stub != IntPtr.Zero)
                        Interop.LibRPCPort.Stub.Destroy(_stub);
                    disposedValue = true;
                }
            }

            ~$$() {
                Dispose(false);
            }

            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }
            #endregion
)__cs_cb";
