/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/structure.h"

class StructureTest : public testing::Test {
 public:
  tidl::Elements* elms;

  virtual void SetUp() {
    elms = new tidl::Elements();
    EXPECT_NE(elms, nullptr);
    elms->Add(new tidl::Element("test1", new tidl::BaseType("int", ""),
        "", __LINE__));
    elms->Add(new tidl::Element("test2", new tidl::BaseType("char *", ""),
        "", __LINE__));
  }
  virtual void TearDown() {}
};

TEST_F(StructureTest, Structure_Constructor) {
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      "", __LINE__);
  EXPECT_NE(structure, nullptr);
  delete structure;
}

TEST_F(StructureTest, Structure_GetElements) {
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      "", __LINE__);
  EXPECT_NE(structure, nullptr);

  int count = 0;
  for (auto& elm : elms->GetElms()) {
    if (elm->GetID() == "test1" &&
        elm->GetType().ToString() == "int")
      count++;
    if (elm->GetID() == "test2" &&
        elm->GetType().ToString() == "char *")
      count++;
  }
  EXPECT_EQ(count, 2);
  delete structure;
}

TEST_F(StructureTest, Structure_GetID) {
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      "", __LINE__);
  EXPECT_NE(structure, nullptr);
  EXPECT_EQ(structure->GetID(), "TestStructure");
  delete structure;
}

TEST_F(StructureTest, Structure_GetType) {
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      "", __LINE__);
  EXPECT_NE(structure, nullptr);
  EXPECT_EQ(structure->GetType(), tidl::Structure::TYPE_STRUCTURE);
  delete structure;
}

TEST_F(StructureTest, Structure_GetLine) {
  unsigned line = __LINE__;
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      "", line);
  EXPECT_NE(structure, nullptr);
  EXPECT_EQ(structure->GetLine(), line);
  delete structure;
}

TEST_F(StructureTest, Structure_GetComments) {
  std::string comments = "Test Structure";
  tidl::Structure* structure = new tidl::Structure("TestStructure", elms,
      comments, __LINE__);
  EXPECT_NE(structure, nullptr);
  EXPECT_EQ(structure->GetComments(), comments);
  delete structure;
}
